@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013 Red Hat Inc.
+ * Copyright (C) 2013-2018 Red Hat Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,6 +42,10 @@
 #include <sys/stat.h>
 #include <errno.h>
 
+#if defined(__linux__) && !defined(FALLOC_FL_PUNCH_HOLE)
+#include <linux/falloc.h>   /* For FALLOC_FL_*, glibc < 2.18 */
+#endif
+
 #include <nbdkit-plugin.h>
 
 #ifndef O_CLOEXEC
@@ -50,6 +54,20 @@
 
 static char *filename = NULL;
 
+#if defined(FALLOC_FL_PUNCH_HOLE) || defined(FALLOC_FL_ZERO_RANGE)
+static int
+do_fallocate(int fd, int mode, off_t offset, off_t len)
+{
+  int r = fallocate (fd, mode, offset, len);
+  if (r == -1 && errno == ENODEV) {
+    /* kernel 3.10 fails with ENODEV for block device. Kernel >= 4.9 fails
+       with EOPNOTSUPP in this case. Normalize errno to simplify callers. */
+    errno = EOPNOTSUPP;
+  }
+  return r;
+}
+#endif
+
 static void
 file_unload (void)
 {
@@ -65,7 +83,7 @@ file_config (const char *key, const char *value)
   if (strcmp (key, "file") == 0) {
     /* See FILENAMES AND PATHS in nbdkit-plugin(3). */
     free (filename);
-    filename = nbdkit_absolute_path (value);
+    filename = nbdkit_realpath (value);
     if (!filename)
       return -1;
   }
@@ -90,10 +108,6 @@ file_config_complete (void)
     nbdkit_error ("you must supply the file=<FILENAME> parameter after the plugin name on the command line");
     return -1;
   }
-  if (access (filename, F_OK) < 0) {
-    nbdkit_error ("access '%s': %m", filename);
-    return -1;
-  }
 
   return 0;
 }
@@ -175,6 +189,21 @@ file_get_size (void *handle)
   return statbuf.st_size;
 }
 
+static int
+file_can_trim (void *handle)
+{
+  /* Trim is advisory, but we prefer to advertise it only when we can
+   * actually (attempt to) punch holes.  Since not all filesystems
+   * support all fallocate modes, it would be nice if we had a way
+   * from fpathconf() to definitively learn what will work on a given
+   * fd for a more precise answer; oh well.  */
+#ifdef FALLOC_FL_PUNCH_HOLE
+  return 1;
+#else
+  return 0;
+#endif
+}
+
 /* Read data from the file. */
 static int
 file_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
@@ -219,7 +248,7 @@ file_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset)
   return 0;
 }
 
-/* Write data to the file. */
+/* Write zeroes to the file. */
 static int
 file_zero (void *handle, uint32_t count, uint64_t offset, int may_trim)
 {
@@ -230,8 +259,8 @@ file_zero (void *handle, uint32_t count, uint64_t offset, int may_trim)
 
 #ifdef FALLOC_FL_PUNCH_HOLE
   if (may_trim) {
-    r = fallocate (h->fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
-		   offset, count);
+    r = do_fallocate (h->fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
+                      offset, count);
     if (r == -1 && errno != EOPNOTSUPP) {
       nbdkit_error ("zero: %m");
     }
@@ -242,7 +271,7 @@ file_zero (void *handle, uint32_t count, uint64_t offset, int may_trim)
 #endif
 
 #ifdef FALLOC_FL_ZERO_RANGE
-  r = fallocate (h->fd, FALLOC_FL_ZERO_RANGE, offset, count);
+  r = do_fallocate (h->fd, FALLOC_FL_ZERO_RANGE, offset, count);
   if (r == -1 && errno != EOPNOTSUPP) {
     nbdkit_error ("zero: %m");
   }
@@ -268,6 +297,33 @@ file_flush (void *handle)
   return 0;
 }
 
+/* Punch a hole in the file. */
+static int
+file_trim (void *handle, uint32_t count, uint64_t offset)
+{
+  int r = -1;
+#ifdef FALLOC_FL_PUNCH_HOLE
+  struct handle *h = handle;
+
+  /* Trim is advisory; we don't care if it fails for anything other
+   * than EIO or EPERM. */
+  r = do_fallocate (h->fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
+                    offset, count);
+  if (r < 0) {
+    if (errno != EPERM && errno != EIO) {
+      nbdkit_debug ("ignoring failed fallocate during trim: %m");
+      r = 0;
+    }
+    else
+      nbdkit_error ("fallocate: %m");
+  }
+#else
+  /* Based on .can_trim, this should not be reached. */
+  errno = EOPNOTSUPP;
+#endif
+  return r;
+}
+
 static struct nbdkit_plugin plugin = {
   .name              = "file",
   .longname          = "nbdkit file plugin",
@@ -279,10 +335,12 @@ static struct nbdkit_plugin plugin = {
   .open              = file_open,
   .close             = file_close,
   .get_size          = file_get_size,
+  .can_trim          = file_can_trim,
   .pread             = file_pread,
   .pwrite            = file_pwrite,
-  .zero              = file_zero,
   .flush             = file_flush,
+  .trim              = file_trim,
+  .zero              = file_zero,
   .errno_is_preserved = 1,
 };
 
