@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2017 Red Hat Inc.
+ * Copyright (C) 2013-2018 Red Hat Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,30 +38,10 @@
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
-#include <assert.h>
-#include <pthread.h>
 
-#include "nbdkit-plugin.h"
 #include "internal.h"
 
-/* Used to group piecemeal message construction into atomic output. */
-static pthread_mutex_t errors_lock = PTHREAD_MUTEX_INITIALIZER;
-
-static void
-lock (void)
-{
-  int r = pthread_mutex_lock (&errors_lock);
-  assert (!r);
-}
-
-static void
-unlock (void)
-{
-  int r = pthread_mutex_unlock (&errors_lock);
-  assert (!r);
-}
-
-/* Called with lock taken. */
+/* Called with flockfile (stderr) taken. */
 static void
 prologue (const char *type)
 {
@@ -88,14 +68,13 @@ nbdkit_vdebug (const char *fs, va_list args)
 
   if (!verbose)
     return;
-
-  lock ();
+  flockfile (stderr);
   prologue ("debug");
 
   vfprintf (stderr, fs, args);
 
   fprintf (stderr, "\n");
-  unlock ();
+  funlockfile (stderr);
 
   errno = err;
 }
@@ -110,7 +89,7 @@ nbdkit_debug (const char *fs, ...)
   if (!verbose)
     return;
 
-  lock ();
+  flockfile (stderr);
   prologue ("debug");
 
   va_start (args, fs);
@@ -118,7 +97,7 @@ nbdkit_debug (const char *fs, ...)
   va_end (args);
 
   fprintf (stderr, "\n");
-  unlock ();
+  funlockfile (stderr);
 
   errno = err;
 }
@@ -129,13 +108,13 @@ nbdkit_verror (const char *fs, va_list args)
 {
   int err = errno;
 
-  lock ();
+  flockfile (stderr);
   prologue ("error");
 
   vfprintf (stderr, fs, args);
 
   fprintf (stderr, "\n");
-  unlock ();
+  funlockfile (stderr);
 
   errno = err;
 }
@@ -147,7 +126,7 @@ nbdkit_error (const char *fs, ...)
   va_list args;
   int err = errno;
 
-  lock ();
+  flockfile (stderr);
   prologue ("error");
 
   va_start (args, fs);
@@ -155,7 +134,7 @@ nbdkit_error (const char *fs, ...)
   va_end (args);
 
   fprintf (stderr, "\n");
-  unlock ();
+  funlockfile (stderr);
 
   errno = err;
 }
