@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013 Red Hat Inc.
+ * Copyright (C) 2013-2018 Red Hat Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,13 +36,18 @@
 
 #include <stdint.h>
 
+/* Note that all NBD fields are sent on the wire in network byte
+ * order, so we must use beXXtoh or htobeXX when reading or writing
+ * these structures.
+ */
+
 /* Old-style handshake. */
 struct old_handshake {
   char nbdmagic[8];           /* "NBDMAGIC" */
-  uint64_t version;           /* OLD_VERSION, in network byte order */
-  uint64_t exportsize;        /* in network byte order */
-  uint16_t gflags;            /* global flags, in network byte order */
-  uint16_t eflags;            /* per-export flags, in network byte order */
+  uint64_t version;           /* OLD_VERSION */
+  uint64_t exportsize;
+  uint16_t gflags;            /* global flags */
+  uint16_t eflags;            /* per-export flags */
   char zeroes[124];           /* must be sent as zero bytes */
 } __attribute__((packed));
 
@@ -51,15 +56,15 @@ struct old_handshake {
 /* New-style handshake. */
 struct new_handshake {
   char nbdmagic[8];           /* "NBDMAGIC" */
-  uint64_t version;           /* NEW_VERSION, in network byte order */
-  uint16_t gflags;            /* global flags, in network byte order */
+  uint64_t version;           /* NEW_VERSION */
+  uint16_t gflags;            /* global flags */
 } __attribute__((packed));
 
 #define NEW_VERSION UINT64_C(0x49484156454F5054)
 
 /* New-style handshake option (sent by the client to us). */
 struct new_option {
-  uint64_t version;           /* NEW_VERSION, in network byte order */
+  uint64_t version;           /* NEW_VERSION */
   uint32_t option;            /* NBD_OPT_* */
   uint32_t optlen;            /* option data length */
   /* option data follows */
@@ -67,7 +72,7 @@ struct new_option {
 
 /* Fixed newstyle handshake reply message. */
 struct fixed_new_option_reply {
-  uint64_t magic;             /* NBD_REP_MAGIC, network byte order */
+  uint64_t magic;             /* NBD_REP_MAGIC */
   uint32_t option;            /* option we are replying to */
   uint32_t reply;             /* NBD_REP_* */
   uint32_t replylen;
@@ -77,26 +82,29 @@ struct fixed_new_option_reply {
 
 /* New-style handshake server reply. */
 struct new_handshake_finish {
-  uint64_t exportsize;        /* in network byte order */
-  uint16_t eflags;            /* per-export flags, in network byte order */
+  uint64_t exportsize;
+  uint16_t eflags;            /* per-export flags */
   char zeroes[124];           /* must be sent as zero bytes */
 } __attribute__((packed));
 
 /* Global flags. */
 #define NBD_FLAG_FIXED_NEWSTYLE 1
+#define NBD_FLAG_NO_ZEROES      2
 
 /* Per-export flags. */
-#define NBD_FLAG_HAS_FLAGS   1
-#define NBD_FLAG_READ_ONLY   2
-#define NBD_FLAG_SEND_FLUSH  4
-#define NBD_FLAG_SEND_FUA    8
-#define NBD_FLAG_ROTATIONAL 16
-#define NBD_FLAG_SEND_TRIM  32
+#define NBD_FLAG_HAS_FLAGS         (1 << 0)
+#define NBD_FLAG_READ_ONLY         (1 << 1)
+#define NBD_FLAG_SEND_FLUSH        (1 << 2)
+#define NBD_FLAG_SEND_FUA          (1 << 3)
+#define NBD_FLAG_ROTATIONAL        (1 << 4)
+#define NBD_FLAG_SEND_TRIM         (1 << 5)
+#define NBD_FLAG_SEND_WRITE_ZEROES (1 << 6)
 
 /* NBD options (new style handshake only). */
 #define NBD_OPT_EXPORT_NAME  1
 #define NBD_OPT_ABORT        2
 #define NBD_OPT_LIST         3
+#define NBD_OPT_STARTTLS     5
 
 #define NBD_REP_ACK          1
 #define NBD_REP_SERVER       2
@@ -104,11 +112,13 @@ struct new_handshake_finish {
 #define NBD_REP_ERR_POLICY   0x80000002
 #define NBD_REP_ERR_INVALID  0x80000003
 #define NBD_REP_ERR_PLATFORM 0x80000004
+#define NBD_REP_ERR_TLS_REQD 0x80000005
 
 /* Request (client -> server). */
 struct request {
   uint32_t magic;               /* NBD_REQUEST_MAGIC. */
-  uint32_t type;                /* Request type. */
+  uint16_t flags;               /* Request flags. */
+  uint16_t type;                /* Request type. */
   uint64_t handle;              /* Opaque handle. */
   uint64_t offset;              /* Request offset. */
   uint32_t count;               /* Request length. */
@@ -129,8 +139,9 @@ struct reply {
 #define NBD_CMD_DISC              2 /* Disconnect. */
 #define NBD_CMD_FLUSH             3
 #define NBD_CMD_TRIM              4
-#define NBD_CMD_MASK_COMMAND 0xffff
-#define NBD_CMD_FLAG_FUA    (1<<16)
+#define NBD_CMD_WRITE_ZEROES      6
+#define NBD_CMD_FLAG_FUA      (1<<0)
+#define NBD_CMD_FLAG_NO_HOLE  (1<<1)
 
 /* Error codes (previously errno).
  * See http://git.qemu.org/?p=qemu.git;a=commitdiff;h=ca4414804114fd0095b317785bc0b51862e62ebb
@@ -141,5 +152,6 @@ struct reply {
 #define NBD_ENOMEM     12
 #define NBD_EINVAL     22
 #define NBD_ENOSPC     28
+#define NBD_ESHUTDOWN 108
 
 #endif /* NBDKIT_PROTOCOL_H */
