@@ -32,8 +32,10 @@
 # SUCH DAMAGE.
 
 set -e
+set -x
 source ./functions.sh
 
+# Basic check that the name field is present.
 output="$(nbdkit example1 --dump-plugin)"
 if [[ ! ( "$output" =~ name\=example1 ) ]]; then
     echo "$0: unexpected output from nbdkit example1 --dump-plugin"
@@ -41,9 +43,33 @@ if [[ ! ( "$output" =~ name\=example1 ) ]]; then
     exit 1
 fi
 
+# example2 overrides the --dump-plugin with extra data.
 output="$(nbdkit example2 --dump-plugin)"
 if [[ ! ( "$output" =~ example2_extra\=hello ) ]]; then
     echo "$0: unexpected output from nbdkit example2 --dump-plugin"
     echo "$output"
     exit 1
 fi
+
+# Get a list of all plugins and run --dump-plugin on all of them.
+# However some of these tests are expected to fail.
+plugins="$(
+    cd ..;
+    ls -1 plugins/*/.libs/nbdkit-*-plugin.so plugins/*/nbdkit-*-plugin |
+        sed 's,^plugins/\([^/]*\)/.*,\1,'
+)"
+for p in $plugins; do
+    case "$p${NBDKIT_VALGRIND:+-valgrind}" in
+        vddk | vddk-valgrind)
+            # VDDK won't run without special environment variables
+            # being set, so ignore it.
+            ;;
+        perl-valgrind | python-valgrind | ruby-valgrind | tcl-valgrind | \
+        example4-valgrind | tar-valgrind)
+            # Plugins written in scripting languages can't run under valgrind.
+            ;;
+        *)
+            nbdkit $p --dump-plugin
+            ;;
+    esac
+done
